/**
 * @file instructions.h
 * Contains list of instructions and adressing modes
 */
#ifndef INSTRUCTIONS_H
#define INSTRUCTIONS_H

/** @brief Enum of adressing modes */
typedef enum {
	IMPL, ACC, IMM, ABS, ABSX, ABSY, ZP, ZPX, ZPY, REL, IND, INDX, INDY
} AdrType;

/** @brief Structure of one instruction entry in the @a INSTRUCTIONS list */
typedef struct _Instr {
	char* name;
	AdrType aType;
	unsigned char len;
} Instr;

/** @brief List of instructions with theirs adressing modes and lengths */
const Instr INSTRUCTIONS[256] = {
	{ "BRK", IMPL, 1 },
	{ "ORA", INDX, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "ORA", ZP, 2 },
	{ "ASL", ZP, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "PHP", IMPL, 1 },
	{ "ORA", IMM, 2 },
	{ "ASL", ACC, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "ORA", ABS, 3 },
	{ "ASL", ABS, 3 },
	{ "UNKNOWN", IMPL, 1 },
	{ "BPL", REL, 2 },
	{ "ORA", INDY, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "ORA", ZPX, 2 },
	{ "ASL", ZPX, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "CLC", IMPL, 1 },
	{ "ORA", ABSY, 3 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "ORA", ABSX, 3 },
	{ "ASL", ABSX, 3 },
	{ "UNKNOWN", IMPL, 1 },
	{ "JSR", ABS, 3 },
	{ "AND", INDX, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "BIT", ZP, 2 },
	{ "AND", ZP, 2 },
	{ "ROL", ZP, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "PLP", IMPL, 1 },
	{ "AND", IMM, 2 },
	{ "ROL", ACC, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "BIT", ABS, 3 },
	{ "AND", ABS, 3 },
	{ "ROL", ABS, 3 },
	{ "UNKNOWN", IMPL, 1 },
	{ "BMI", REL, 2 },
	{ "AND", INDY, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "AND", ZPX, 2 },
	{ "ROL", ZPX, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "SEC", IMPL, 1 },
	{ "AND", ABSY, 3 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "AND", ABSX, 3 },
	{ "ROL", ABSX, 3 },
	{ "UNKNOWN", IMPL, 1 },
	{ "RTI", IMPL, 1 },
	{ "EOR", INDX, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "EOR", ZP, 2 },
	{ "LSR", ZP, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "PHA", IMPL, 1 },
	{ "EOR", IMM, 2 },
	{ "LSR", ACC, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "JMP", ABS, 3 },
	{ "EOR", ABS, 3 },
	{ "LSR", ABS, 3 },
	{ "UNKNOWN", IMPL, 1 },
	{ "BVC", REL, 2 },
	{ "EOR", INDY, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "EOR", ZPX, 2 },
	{ "LSR", ZPX, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "CLI", IMPL, 1 },
	{ "EOR", ABSY, 3 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "EOR", ABSX, 3 },
	{ "LSR", ABSX, 3 },
	{ "UNKNOWN", IMPL, 1 },
	{ "RTS", IMPL, 1 },
	{ "ADC", INDX, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "ADC", ZP, 2 },
	{ "ROR", ZP, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "PLA", IMPL, 1 },
	{ "ADC", IMM, 2 },
	{ "ROR", ACC, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "JMP", IND, 3 },
	{ "ADC", ABS, 3 },
	{ "ROR", ABS, 3 },
	{ "UNKNOWN", IMPL, 1 },
	{ "BVS", REL, 2 },
	{ "ADC", INDY, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "ADC", ZPX, 2 },
	{ "ROR", ZPX, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "SEI", IMPL, 1 },
	{ "ADC", ABSY, 3 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "ADC", ABSX, 3 },
	{ "ROR", ABSX, 3 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "STA", INDX, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "STY", ZP, 2 },
	{ "STA", ZP, 2 },
	{ "STX", ZP, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "DEY", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "TXA", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "STY", ABS, 3 },
	{ "STA", ABS, 3 },
	{ "STX", ABS, 3 },
	{ "UNKNOWN", IMPL, 1 },
	{ "BCC", REL, 2 },
	{ "STA", INDY, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "STY", ZPX, 2 },
	{ "STA", ZPX, 2 },
	{ "STX", ZPX, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "TYA", IMPL, 1 },
	{ "STA", ABSY, 3 },
	{ "TXS", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "STA", ABSX, 3 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "LDY", IMM, 2 },
	{ "LDA", INDX, 2 },
	{ "LDX", IMM, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "LDY", ZP, 2 },
	{ "LDA", ZP, 2 },
	{ "LDX", ZP, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "TAY", IMPL, 1 },
	{ "LDA", IMM, 2 },
	{ "TAX", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "LDY", ABS, 3 },
	{ "LDA", ABS, 3 },
	{ "LDX", ABS, 3 },
	{ "BCS", REL, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "LDA", INDY, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "LDY", ZPX, 2 },
	{ "LDA", ZPX, 2 },
	{ "LDX", ZPY, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "CLV", IMPL, 1 },
	{ "LDA", ABSY, 3 },
	{ "TSX", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "LDY", ABSX, 3 },
	{ "LDA", ABSX, 3 },
	{ "LDX", ABSY, 3 },
	{ "UNKNOWN", IMPL, 1 },
	{ "CPY", IMM, 2 },
	{ "CMP", INDX, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "CPY", ZP, 2 },
	{ "CMP", ZP, 2 },
	{ "DEC", ZP, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "INY", IMPL, 1 },
	{ "CMP", IMM, 2 },
	{ "DEX", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "CPY", ABS, 3 },
	{ "CMP", ABS, 3 },
	{ "DEC", ABS, 3 },
	{ "UNKNOWN", IMPL, 1 },
	{ "BNE", REL, 2 },
	{ "CMP", INDY, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "CMP", ZPX, 2 },
	{ "DEC", ZPX, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "CLD", IMPL, 1 },
	{ "CMP", ABSY, 3 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "CMP", ABSX, 3 },
	{ "DEC", ABSX, 3 },
	{ "UNKNOWN", IMPL, 1 },
	{ "CPX", IMM, 2 },
	{ "SBC", INDX, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "CPX", ZP, 2 },
	{ "SBC", ZP, 2 },
	{ "INC", ZP, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "INX", IMPL, 1 },
	{ "SBC", IMM, 2 },
	{ "NOP", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "CPX", ABS, 3 },
	{ "SBC", ABS, 3 },
	{ "INC", ABS, 3 },
	{ "UNKNOWN", IMPL, 1 },
	{ "BEQ", REL, 2 },
	{ "SBC", INDY, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "SBC", ZPX, 2 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "SED", IMPL, 1 },
	{ "SBC", ABSY, 3 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "UNKNOWN", IMPL, 1 },
	{ "SBC", ABSX, 3 },
	{ "INC", ABSX, 3 },
	{ "UNKNOWN", IMPL, 1 }
};

#endif INSTRUCTIONS_H